@@ -2,7 +2,8 @@
 
 ###########################################################################
 #    OCRFeeder - The complete OCR suite
-#    Copyright (C) 2009 Joaquim Rocha
+#    Copyright (C) 2009-2013 Joaquim Rocha
+#    Copyright (C) 2009-2012 Igalia, S.L.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -24,10 +25,9 @@ import sys
 import os.path
 import urllib
 import widgetPresenter
-from widgetModeler import SourceImagesListStore, \
-     SourceImagesSelectorIconView, ImageReviewer_Controler
+from widgetModeler import ImageReviewer_Controler
 from dataHolder import DataBox, TextData
-from customWidgets import SelectableBoxesArea
+from pagesiconview import PagesIconView
 from ocrfeeder.feeder.ocrEngines import Engine, OcrEnginesManager
 from ocrfeeder.feeder.documentGeneration import DocumentGeneratorManager
 from ocrfeeder.util.configuration import ConfigurationManager
@@ -35,12 +35,9 @@ from ocrfeeder.util.asyncworker import AsyncItem
 from optparse import OptionParser
 import gettext
 import locale
+from gi.repository import Gdk, Gtk
 _ = gettext.gettext
 
-import pygtk
-pygtk.require('2.0')
-import gtk
-
 class Studio:
 
     TARGET_TYPE_URI_LIST = 80
@@ -75,22 +72,17 @@ class Studio:
         self.engines_needing_update = \
             self.ocr_engines_manager.makeEnginesFromFolder(user_engines_folder)
         self.ocr_engines = self.ocr_engines_manager.ocr_engines
-        self.source_images_list_store = SourceImagesListStore()
-        self.source_images_icon_view = SourceImagesSelectorIconView(self.source_images_list_store)
-        self.source_images_icon_view.setDeleteCurrentPageFunction(self.deleteCurrentPage)
-        self.source_images_icon_view.connect('drag_data_received', self.dragDataReceived)
-        self.source_images_icon_view.connect('drag_drop', self.dragDrop)
-        self.source_images_icon_view.get_model().connect('row-inserted',
+        self.pages_icon_view = PagesIconView()
+        self.pages_icon_view.setDeleteCurrentPageFunction(self.deleteCurrentPage)
+        self.pages_icon_view.get_model().connect('row-inserted',
                                                  self.__pagesUpdatedCallback)
-        self.source_images_icon_view.get_model().connect('row-deleted',
+        self.pages_icon_view.get_model().connect('row-deleted',
                                                  self.__pagesUpdatedCallback)
-        self.source_images_icon_view.drag_dest_set(gtk.DEST_DEFAULT_MOTION | gtk.DEST_DEFAULT_HIGHLIGHT,
-                                                   [('text/uri-list', 0, self.TARGET_TYPE_URI_LIST)], gtk.gdk.ACTION_COPY)
-        self.source_images_icon_view.show()
-        self.main_window.main_area_left.add_with_viewport(self.source_images_icon_view)
+        self.pages_icon_view.show()
+        self.main_window.main_area_left.add_with_viewport(self.pages_icon_view)
         self.images_selectable_area = {}
         self.source_images_controler = ImageReviewer_Controler(self.main_window,
-                                                               self.source_images_icon_view,
+                                                               self.pages_icon_view,
                                                                self.ocr_engines,
                                                                self.configuration_manager)
         self.project_name = None
@@ -106,7 +98,7 @@ class Studio:
                                  'unpaper': self.unpaper, 'preferences': self.preferences, 'about': self.about,
                                  'ocr_engines': self.ocrEngines, 'zoom_in': self.zoomIn, 'zoom_out': self.zoomOut,
                                  'zoom_fit': self.zoomFit, 'reset_zoom': self.resetZoom,
-                                 'recognize_areas': self.source_images_controler.recognizeSelectedAreas,
+                                 'recognize_areas': self.__recognizeSelectedAreas,
                                  'import_from_scanner': self.importFromScanner,
                                  'select_next_area': self.source_images_controler.selectNextArea,
                                  'select_previous_area': self.source_images_controler.selectPreviousArea,
@@ -146,7 +138,7 @@ class Studio:
         self.main_window.setHasSelectedBoxes(False)
         self.main_window.setHasContentBoxes(False)
         self.main_window.setNumberOfPages(
-            self.source_images_icon_view.getNumberOfPages())
+            self.pages_icon_view.getNumberOfPages())
 
         # Show dialog to choose system-wide OCR engines when no engine was found
         if not self.ocr_engines:
@@ -154,7 +146,7 @@ class Studio:
             if engines:
                 add_engines_dialog = widgetPresenter.SystemEnginesDialog(engines)
                 response = add_engines_dialog.run()
-                if response == gtk.RESPONSE_ACCEPT:
+                if response == Gtk.ResponseType.ACCEPT:
                     for engine in add_engines_dialog.getChosenEngines():
                         self.ocr_engines_manager.addNewEngine(engine)
                 add_engines_dialog.destroy()
@@ -162,49 +154,28 @@ class Studio:
         else:
             self.__askForEnginesMigration()
 
+        self._last_images_dir = ''
+
+        if not self.ocr_engines:
+            self.warnNoOCREngines()
+
     def run(self):
-        gtk.gdk.threads_init()
-        gtk.main()
-
-    def dragDataReceived(self, widget, context, x, y, selection, target_type, timestamp):
-        if target_type == self.TARGET_TYPE_URI_LIST:
-            uris = selection.data.strip('\r\n\x00').split()
-            paths = []
-            for uri in uris:
-                path = ""
-                if uri.startswith('file:\\\\\\'): # windows
-                    path = uri[8:]
-                elif uri.startswith('file://'): # nautilus, rox
-                    path = uri[7:]
-                elif uri.startswith('file:'): # xffm
-                    path = uri[5:]
-                path = urllib.url2pathname(path).strip('\r\n\x00')
-                if os.path.isfile(path):
-                    paths.append(path)
-            for path in paths:
-                if os.path.splitext(path)[1] == '.pdf':
-                    folder = lib.convertPdfToImages(path,
-                                       self.configuration_manager.TEMPORARY_FOLDER)
-                    self.__addImagesToReviewer(lib.getImagesFromFolder(folder))
-                else:
-                    try:
-                        self.__addImagesToReviewer([path])
-                    except:
-                        pass
-
-    def dragDrop(self, widget, context, x, y, timestamp):
-        target_atom = widget.drag_dest_find_target(context, widget.drag_dest_get_target_list())
-        if target_atom != None:
-            widget.drag_get_data(context, target_atom, timestamp)
-            context.finish(True, False, timestamp)
-        return True
+        Gdk.threads_init()
+        Gtk.main()
 
     def addImage(self, widget):
         file_open_dialog = widgetPresenter.FileDialog('open', file_filters = [(_('Images'), ['image/*'], [])])
         file_open_dialog.set_select_multiple(True)
+
+        if self._last_images_dir:
+            file_open_dialog.set_current_folder(self._last_images_dir)
+
         response = file_open_dialog.run()
-        if response == gtk.RESPONSE_OK:
+
+        if response == Gtk.ResponseType.OK:
             self.__addImagesToReviewer(file_open_dialog.get_filenames())
+
+        self._last_images_dir = file_open_dialog.get_current_folder()
         file_open_dialog.destroy()
 
     def importFromScanner(self, widget):
@@ -223,11 +194,11 @@ class Studio:
             scanner_chooser_dialog = widgetPresenter.ScannerChooserDialog(\
                                                     self.main_window.window,
                                                     devices)
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
             response = scanner_chooser_dialog.run()
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
             scanner_chooser_dialog.destroy()
-            if response == gtk.RESPONSE_ACCEPT:
+            if response == Gtk.ResponseType.ACCEPT:
                 device = scanner_chooser_dialog.getSelectedDevice()
             else:
                 return
@@ -246,30 +217,30 @@ class Studio:
                                              _("No scanner devices were found"),
                                              _("Error"),
                                              'warning')
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
             error.run()
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
 
     def __scanFinishedCb(self, dialog, image_path, error):
         dialog.destroy()
         if image_path:
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
             self.__addImagesToReviewer([image_path])
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
         else:
             error = widgetPresenter.SimpleDialog(\
                                              _("Error scanning page"),
                                              _("Error"),
                                              'warning')
-            gtk.gdk.threads_enter()
+            Gdk.threads_enter()
             error.run()
-            gtk.gdk.threads_leave()
+            Gdk.threads_leave()
 
     def importPdf(self, widget):
         file_open_dialog = widgetPresenter.FileDialog('open', file_filters = [(_('PDF'), ['application/pdf'], [])])
         response = file_open_dialog.run()
         files = []
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             files = file_open_dialog.get_filenames()
         file_open_dialog.destroy()
         for file_name in files:
@@ -291,7 +262,7 @@ class Studio:
     def addFolder(self, widget):
         file_open_dialog = widgetPresenter.FileDialog('select-folder')
         response = file_open_dialog.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             filenames = file_open_dialog.get_filenames()
             file_open_dialog.destroy()
             for folder in filenames:
@@ -311,7 +282,7 @@ class Studio:
         export_dialog = widgetPresenter.ExportDialog(_('Export pages'),
                                                      format_names)
         response = export_dialog.run()
-        if response == gtk.RESPONSE_ACCEPT:
+        if response == Gtk.ResponseType.ACCEPT:
             format = export_dialog.getSelectedFormat()
             export_dialog.destroy()
             if format != -1:
@@ -328,22 +299,22 @@ class Studio:
     def deleteCurrentPage(self, widget):
         delete_dialog = widgetPresenter.QuestionDialog(_('Are you sure you want to delete the current image?'))
         response = delete_dialog.run()
-        if response == gtk.RESPONSE_YES:
+        if response == Gtk.ResponseType.YES:
             self.source_images_controler.deleteCurrentPage()
-            self.source_images_icon_view.deleteCurrentSelection()
+            self.pages_icon_view.deleteCurrentSelection()
         delete_dialog.destroy()
 
     def movePageDown(self, widget):
-        self.source_images_icon_view.movePage(1)
+        self.pages_icon_view.movePage(1)
 
     def movePageUp(self, widget):
-        self.source_images_icon_view.movePage(-1)
+        self.pages_icon_view.movePage(-1)
 
     def selectNextPage(self, widget):
-        self.source_images_icon_view.selectPageFromOffset(1)
+        self.pages_icon_view.selectPageFromOffset(1)
 
     def selectPreviousPage(self, widget):
-        self.source_images_icon_view.selectPageFromOffset(-1)
+        self.pages_icon_view.selectPageFromOffset(-1)
 
     def __addImagesToReviewer(self, images):
         if not images:
@@ -351,11 +322,23 @@ class Studio:
         self.source_images_controler.addImages(images)
 
     def __recognizePageAction(self, widget):
+        if not self.ocr_engines_manager.ocr_engines:
+            self.warnNoOCREngines()
+            return
         self.source_images_controler.recognizeCurrentPage()
 
     def __recognizeDocumentAction(self, widget):
+        if not self.ocr_engines_manager.ocr_engines:
+            self.warnNoOCREngines()
+            return
         self.source_images_controler.recognizeDocument()
 
+    def __recognizeSelectedAreas(self, widget):
+        if not self.ocr_engines_manager.ocr_engines:
+            self.warnNoOCREngines()
+            return
+        self.source_images_controler.recognizeSelectedAreas()
+
     def setProjectName(self, project_name):
         self.project_name = project_name
         project_title = os.path.splitext(os.path.basename(self.project_name))[0]
@@ -388,7 +371,7 @@ class Studio:
         dialog = widgetPresenter.QuestionDialog(_('Are you sure you want '
                                                   'to clear the project?'))
         response = dialog.run()
-        if response == gtk.RESPONSE_YES:
+        if response == Gtk.ResponseType.YES:
             self.source_images_controler.clear()
         dialog.destroy()
 
@@ -400,7 +383,7 @@ class Studio:
 
     def preferences(self, widget = None):
         preferences_dialog = widgetPresenter.PreferencesDialog(self.configuration_manager, self.ocr_engines_manager.ocr_engines)
-        if preferences_dialog.run() == gtk.RESPONSE_ACCEPT:
+        if preferences_dialog.run() == Gtk.ResponseType.ACCEPT:
             preferences_dialog.saveToManager()
             self.source_images_controler.updateFromConfiguration()
         preferences_dialog.destroy()
@@ -408,8 +391,31 @@ class Studio:
     def ocrEngines(self, widget = None):
         ocr_dialog = widgetPresenter.OcrManagerDialog(self.ocr_engines_manager)
         ocr_dialog.run()
+        if ocr_dialog.modified:
+            ocr_engines = self.ocr_engines_manager.ocr_engines
+            self.source_images_controler.updateOcrEngines(ocr_engines)
         ocr_dialog.destroy()
 
+    def warnNoOCREngines(self):
+        lib.debug('No OCR engines found')
+        dialog = Gtk.MessageDialog(self.main_window.window,
+                                   Gtk.DialogFlags.MODAL |
+                                   Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                   Gtk.MessageType.INFO)
+        dialog.set_title(_('No OCR engines available'))
+        dialog.set_markup(_('No OCR engines were found in the system.\n'
+                            'Please make sure you have OCR engines installed '
+                            'and available.'))
+        dialog.add_buttons(_('_Ignore'),
+                           Gtk.ResponseType.CANCEL,
+                           _('_Open OCR Engines Manager Dialog'),
+                           Gtk.ResponseType.OK)
+
+        response = dialog.run()
+        dialog.destroy()
+        if response == Gtk.ResponseType.OK:
+            self.ocrEngines()
+
     def enginesTool(self, widget = None):
         pass
 
@@ -419,9 +425,9 @@ class Studio:
             about_dialog.destroy()
 
     def showHelpContents(self, widget = None):
-        gtk.show_uri(self.main_window.window.get_screen(),
+        Gtk.show_uri(self.main_window.window.get_screen(),
                      'ghelp:ocrfeeder',
-                     gtk.get_current_event_time())
+                     Gtk.get_current_event_time())
 
     def zoomIn(self, widget = None):
         self.source_images_controler.zoomIn()
@@ -437,7 +443,7 @@ class Studio:
 
     def __pagesUpdatedCallback(self, model, path, iter = None):
         self.main_window.setNumberOfPages(
-            self.source_images_icon_view.getNumberOfPages())
+            self.pages_icon_view.getNumberOfPages())
 
     def __askForEnginesMigration(self):
         auto_update = self.engines_needing_update['auto']
@@ -451,14 +457,14 @@ class Studio:
             names = []
             for migration in manual_update:
                 names.append(migration['engine'].name)
-            dialog = gtk.MessageDialog(self.main_window.window,
-                                       gtk.DIALOG_MODAL |
-                                       gtk.DIALOG_DESTROY_WITH_PARENT,
-                                       gtk.MESSAGE_WARNING)
+            dialog = Gtk.MessageDialog(self.main_window.window,
+                                       Gtk.DialogFlags.MODAL |
+                                       Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                                       Gtk.MessageType.WARNING)
             dialog.add_buttons(_('_Keep Current Configuration') ,
-                               gtk.RESPONSE_CANCEL,
+                               Gtk.ResponseType.CANCEL,
                                _('_Open OCR Engines Manager Dialog'),
-                               gtk.RESPONSE_OK)
+                               Gtk.ResponseType.OK)
             message = _('The following OCR engines\' arguments '
                         'might need to be updated but it appears '
                         'you have changed their default configuration so '
@@ -474,20 +480,20 @@ class Studio:
             for migration in manual_update:
                 self.ocr_engines_manager.migrateEngine(migration['engine'],
                                                 migration['configuration'])
-            if response == gtk.RESPONSE_OK:
+            if response == Gtk.ResponseType.OK:
                 self.ocrEngines()
 
     def quit(self, widget = None, data = None):
-        if not self.project_name and not self.source_images_list_store.isEmpty():
-            quit_dialog = widgetPresenter.QuestionDialog('<b>' + _("The project hasn't been saved.") + '</b>', gtk.BUTTONS_NONE)
+        if not self.project_name and not self.pages_icon_view.isEmpty():
+            quit_dialog = widgetPresenter.QuestionDialog('<b>' + _("The project hasn't been saved.") + '</b>', Gtk.ButtonsType.NONE)
             quit_dialog.format_secondary_text(_('Do you want to save it before closing?'))
-            quit_dialog.add_buttons(_('Close anyway'), gtk.RESPONSE_NO, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE_AS, gtk.RESPONSE_YES)
+            quit_dialog.add_buttons(_('Close anyway'), Gtk.ResponseType.NO, Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE_AS, Gtk.ResponseType.YES)
             response = quit_dialog.run()
             quit_dialog.destroy()
-            if response == gtk.RESPONSE_YES:
+            if response == Gtk.ResponseType.YES:
                 self.saveProjectAs()
                 self.__main_quit()
-            elif response == gtk.RESPONSE_NO:
+            elif response == Gtk.ResponseType.NO:
                 quit_dialog.destroy()
                 self.__main_quit()
             else:
@@ -499,4 +505,4 @@ class Studio:
     def __main_quit(self):
         self.configuration_manager.configurationToXml()
         self.configuration_manager.removeTemporaryFolder()
-        gtk.main_quit()
+        Gtk.main_quit()
