@@ -2,7 +2,7 @@
 
 ###########################################################################
 #    OCRFeeder - The complete OCR suite
-#    Copyright (C) 2009 Joaquim Rocha
+#    Copyright (C) 2009-2013 Joaquim Rocha
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -18,12 +18,11 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ###########################################################################
 
-from customWidgets import SelectableBoxesArea
+from boxesarea import BoxesArea
 from dataHolder import DataBox, PageData, TEXT_TYPE, IMAGE_TYPE
 from ocrfeeder.feeder.documentGeneration import OdtGenerator, HtmlGenerator, PlaintextGenerator, PdfGenerator
 from ocrfeeder.feeder.imageManipulation import *
 from ocrfeeder.feeder.layoutAnalysis import *
-from pango import FontDescription, SCALE
 from project import ProjectSaver, ProjectLoader
 from ocrfeeder.util import graphics, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_CENTER, \
      ALIGN_FILL, PAPER_SIZES
@@ -32,165 +31,17 @@ from ocrfeeder.util.configuration import ConfigurationManager
 from ocrfeeder.util import constants
 from ocrfeeder.util.asyncworker import AsyncItem
 from widgetPresenter import BoxEditor, PagesToExportDialog, FileDialog, \
-    PageSizeDialog, getPopupMenu, WarningDialog, UnpaperDialog, \
+    PageSizeDialog, WarningDialog, UnpaperDialog, \
     QueuedEventsProgressDialog, SpellCheckerDialog
 import gettext
-import gobject
-import gtk
+from gi.repository import Gtk, GObject, Gdk, GdkPixbuf, Pango
 import math
 import os.path
-import pygtk
 import threading
 import sys
-pygtk.require('2.0')
 _ = gettext.gettext
 
-
-
-class SourceImagesListStore(gtk.ListStore):
-
-    def __init__(self, list_of_images = []):
-        super(SourceImagesListStore, self).__init__(str, gtk.gdk.Pixbuf, object)
-        if len(list_of_images):
-            for path in list_of_images:
-                self.__renderImage(path, self.__generateImageName(path))
-
-    def addImage(self, page_data):
-        image_name = self.__generateImageName(page_data.image_path)
-        return self.__renderImage(image_name, page_data)
-
-    def __renderImage(self, image_name, page_data):
-        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(page_data.image_path,
-                                                      150, 100)
-        return self.append([image_name, pixbuf, page_data])
-
-    def __countEqualPathsStored(self, path):
-        iter = self.get_iter_root()
-        counter = 0
-        while iter != None:
-            page_data = self.get_value(iter, 2)
-            image_path = page_data.image_path
-            if image_path == path:
-                counter += 1
-            iter = self.iter_next(iter)
-        return counter
-
-    def __generateImageName(self, path):
-        image_name = os.path.basename(path)
-        number_of_equal_paths = self.__countEqualPathsStored(path)
-        if number_of_equal_paths:
-            image_name += ' ('+ str(number_of_equal_paths + 1) + ')'
-        return image_name
-
-    def getPixbufsSorted(self):
-        pixbufs = []
-        iter = self.get_iter_root()
-        while iter != None:
-            pixbufs.append(self.get_value(iter, 2))
-            iter = self.iter_next(iter)
-        return pixbufs
-
-    def removeIter(self, path):
-        iter = self.get_iter(path)
-        self.remove(iter)
-
-    def isEmpty(self):
-        return self.get_iter_first() == None
-
-class SourceImagesSelectorIconView(gtk.IconView):
-
-    def __init__(self, source_images_list_store):
-        super(SourceImagesSelectorIconView, self).__init__(source_images_list_store)
-        self.get_accessible().set_name(_('Pages'))
-        self.set_text_column(0)
-        self.set_pixbuf_column(1)
-        self.set_orientation(gtk.ORIENTATION_VERTICAL)
-        self.set_columns(1)
-        self.set_reorderable(True)
-        self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
-        self.set_selection_mode(gtk.SELECTION_BROWSE)
-        self.connect('button-press-event', self.pressedRightButton)
-
-    def pressedRightButton(self, target, event):
-        if event.button == 3:
-            selected_items = self.get_selected_items()
-            if selected_items:
-                menu = getPopupMenu([(gtk.STOCK_DELETE, _('Delete'), self.delete_current_page_function)])
-                menu.popup(None, None, None, event.button, event.time)
-
-    def getSelectedPageData(self):
-        selected_items = self.get_selected_items()
-        if len(selected_items):
-            selected_item_path = selected_items[0]
-            model = self.get_model()
-            iter = model.get_iter(selected_item_path)
-            return self.get_model().get_value(iter, 2)
-        return None
-
-    def getAllPages(self):
-        model = self.get_model()
-        pages = []
-        iter = model.get_iter_root()
-        while iter:
-            pages.append(model.get_value(iter, 2))
-            iter = model.iter_next(iter)
-        return pages
-
-    def setDeleteCurrentPageFunction(self, function):
-        self.delete_current_page_function = function
-
-    def deleteCurrentSelection(self):
-        selected_items = self.get_selected_items()
-        if len(selected_items):
-            selected_item_path = selected_items[0]
-            self.get_model().removeIter(selected_item_path)
-            if not self.get_model().isEmpty():
-                self.select_path(0)
-
-    def clear(self):
-        self.get_model().clear()
-
-    def _getIndexFromOffset(self, offset):
-        selected_items = self.get_selected_items()
-        if not len(selected_items):
-            return
-        selected_item_path = selected_items[0]
-        model = self.get_model()
-        iter = model.get_iter(selected_item_path)
-        index = model.get_path(iter)[0] + offset
-        number_of_items = model.iter_n_children(None)
-        if index < 0:
-            index = number_of_items + offset
-        elif index == number_of_items:
-            index = 0
-        return index
-
-    def movePage(self, offset):
-        selected_items = self.get_selected_items()
-        if not len(selected_items):
-            return
-        selected_item_path = selected_items[0]
-        model = self.get_model()
-        index = self._getIndexFromOffset(offset)
-        if index != selected_item_path[0] + offset:
-            return
-        model.swap(model.get_iter((index,)),
-                   model.get_iter(selected_item_path))
-        self.select_path((index,))
-
-    def selectPageFromOffset(self, offset):
-        selected_items = self.get_selected_items()
-        if not len(selected_items):
-            return
-        selected_item_path = selected_items[0]
-        model = self.get_model()
-        index = self._getIndexFromOffset(offset)
-        self.select_path((index,))
-
-    def getNumberOfPages(self):
-        return self.get_model().iter_n_children(None)
-
-class ImageReviewer(gtk.HPaned):
+class ImageReviewer(Gtk.HPaned):
 
     def __init__(self, main_window, page_data, ocr_engines):
         super(ImageReviewer, self).__init__()
@@ -200,24 +51,24 @@ class ImageReviewer(gtk.HPaned):
         self.text_box_fill_color = (94, 156, 235, 150)
         self.box_stroke_color = (94, 156, 235, 250)
         self.image_box_fill_color = (0, 183, 0, 150)
-        self.selectable_boxes_area = SelectableBoxesArea(self.path_to_image)
+        self.selectable_boxes_area = BoxesArea(self.path_to_image)
         self.selectable_boxes_area.connect('selected_box', self.selectedBox)
         self.selectable_boxes_area.connect('removed_box', self.removedBox)
         self.selectable_boxes_area.connect('updated_box', self.updatedBox)
         self.selectable_boxes_area.connect('dragged_box', self.updatedBoxBounds)
         self.selectable_boxes_area.connect('deselected_box',
                                            self.deselectedBoxCb)
-        self.image_pixbuf = gtk.gdk.pixbuf_new_from_file(self.path_to_image)
+        self.image_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.path_to_image)
         self.set_position(500)
         self.show()
         self.ocr_engines = ocr_engines
         self.editor = Editor(self.image_pixbuf, self.ocr_engines, self)
         self.boxes_dict = {}
 
-        selectable_boxes_scrolled_window = gtk.ScrolledWindow()
+        selectable_boxes_scrolled_window = Gtk.ScrolledWindow()
         selectable_boxes_scrolled_window.get_accessible().set_name(
                                                          _('Selectable areas'))
-        selectable_boxes_scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        selectable_boxes_scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         selectable_boxes_scrolled_window.add(self.selectable_boxes_area)
         self.selectable_boxes_area.show()
         selectable_boxes_scrolled_window.show()
@@ -317,10 +168,15 @@ class ImageReviewer(gtk.HPaned):
             engine.setLanguage(data_box.getLanguage())
         pixbuf_width = self.image_pixbuf.get_width()
         pixbuf_height = self.image_pixbuf.get_height()
-        subpixbuf = self.image_pixbuf.subpixbuf(data_box.getX(),
-                                                data_box.getY(),
-                                                min(data_box.getWidth(), pixbuf_width),
-                                                min(data_box.getHeight(), pixbuf_height))
+        new_pixbuf_width = min(data_box.getWidth(), pixbuf_width)
+        new_pixbuf_height = min(data_box.getHeight(), pixbuf_height)
+        subpixbuf = self.image_pixbuf.new_subpixbuf(data_box.getX(),
+                                                    data_box.getY(),
+                                                    new_pixbuf_width,
+                                                    new_pixbuf_height)
+        subpixbuf.x = data_box.getX()
+        subpixbuf.y = data_box.getY()
+        subpixbuf.width = pixbuf_width
         image = graphics.convertPixbufToImage(subpixbuf)
         layout_analysis = LayoutAnalysis(engine,
                                          clean_text = self.configuration_manager.clean_text)
@@ -351,7 +207,7 @@ class ImageReviewer(gtk.HPaned):
     def copyTextToClipboard(self):
         selected_boxes = self.selectable_boxes_area.getSelectedAreas()
         text = self.getTextFromBoxes(selected_boxes)
-        gtk.Clipboard().set_text(text)
+        Gtk.Clipboard.get(Gdk.Atom.intern("CLIPBOARD", True)).set_text(text, -1)
 
     def getAllText(self):
         boxes = self.selectable_boxes_area.getAllAreas()
@@ -388,7 +244,7 @@ class ImageReviewer(gtk.HPaned):
         if not os.path.exists(self.path_to_image):
             return
         try:
-            self.image_pixbuf = gtk.gdk.pixbuf_new_from_file(self.path_to_image)
+            self.image_pixbuf = GdkPixbuf.Pixbuf.new_from_file(self.path_to_image)
         except Exception, exception:
             debug(exception.message)
             return
@@ -403,7 +259,8 @@ class ImageReviewer(gtk.HPaned):
 
     def zoomFit(self):
         parent = self.selectable_boxes_area.get_parent()
-        parent_height, parent_width = parent.allocation.height, parent.allocation.width
+        allocation = parent.get_allocation()
+        parent_height, parent_width = allocation.height, allocation.width
         image_height, image_width = self.selectable_boxes_area.getImageSize()
         changed = False
         if image_height > parent_height:
@@ -430,20 +287,24 @@ class ImageReviewer(gtk.HPaned):
     def focusCurrentEditorTextArea(self):
         self.editor.box_editor.text_widget.grab_focus()
 
+    def updateOcrEngines(self, ocr_engines):
+        self.ocr_engines = ocr_engines
+        self.editor.updateOcrEngines(ocr_engines)
+
 class ImageReviewer_Controler:
 
     REVIEWER_CACHE_LENGTH = 5
 
-    def __init__(self, main_window, source_images_selector_widget,
+    def __init__(self, main_window, pages_icon_view,
                  ocr_engines, configuration_manager):
         self.main_window = main_window
         self.notebook = self.main_window.notebook
-        self.source_images_selector_widget = source_images_selector_widget
+        self.pages_icon_view = pages_icon_view
         self.ocr_engines = ocr_engines
         self.configuration_manager = configuration_manager
         self.statusbar = self.main_window.statusbar
         self._page_info_message_id = self.statusbar.get_context_id('page_info_message')
-        self.source_images_selector_widget.connect('selection-changed', self.selectImageReviewer)
+        self.pages_icon_view.connect('selection-changed', self.selectImageReviewer)
         self.__updateStatusBar()
 
     def __createdImageReviewer(self, page_data):
@@ -456,6 +317,9 @@ class ImageReviewer_Controler:
 
     def addImages(self, image_path_list):
         item_list = []
+        temp_dir = self.configuration_manager.TEMPORARY_FOLDER
+        image_path_list = graphics.convertMultiImagesInList(image_path_list,
+                                                            temp_dir)
         item_list_length = len(image_path_list)
         if not self.configuration_manager.deskew_images_after_addition and \
            not self.configuration_manager.unpaper_images_after_addition:
@@ -506,11 +370,11 @@ class ImageReviewer_Controler:
             return None
         if page_data is None:
             page_data = PageData(image_path)
-        iter = self.source_images_selector_widget.get_model().addImage(page_data)
+        iter = self.pages_icon_view.get_model().addImage(page_data)
         if select_image:
             path = \
-                self.source_images_selector_widget.get_model().get_path(iter)
-            self.source_images_selector_widget.select_path(path)
+                self.pages_icon_view.get_model().get_path(iter)
+            self.pages_icon_view.select_path(path)
         return page_data
 
     def __deskewImage(self, image_path, target_image_path = None):
@@ -544,7 +408,7 @@ class ImageReviewer_Controler:
         dialog.cancel()
 
     def selectImageReviewer(self, widget):
-        page_data = self.source_images_selector_widget.getSelectedPageData()
+        page_data = self.pages_icon_view.getSelectedPageData()
         if not page_data:
             return
         reviewer = self.__setImageReviewerFromPageData(page_data)
@@ -584,7 +448,7 @@ class ImageReviewer_Controler:
         if n_pages == self.REVIEWER_CACHE_LENGTH:
             self.notebook.remove_page(0)
         reviewer = self.__createdImageReviewer(page_data)
-        index = self.notebook.append_page(reviewer)
+        index = self.notebook.append_page(reviewer, None)
         self.notebook.set_current_page(index)
         return reviewer
 
@@ -597,14 +461,16 @@ class ImageReviewer_Controler:
             reviewer.updatePageData(reviewer.page)
             index += 1
 
-    def recognizeSelectedAreas(self, widget):
+    def recognizeSelectedAreas(self):
         image_reviewer = self.__getCurrentReviewer()
         image_reviewer.performOcrForSelectedBoxes()
 
     def __confirmOveritePossibilityByRecognition(self):
-        confirm_recognition = gtk.MessageDialog(parent = self.main_window.window,
-                                                type = gtk.MESSAGE_QUESTION,
-                                                buttons = gtk.BUTTONS_YES_NO)
+        confirm_recognition = Gtk.MessageDialog(self.main_window.window,
+                                                message_type = Gtk.MessageType.QUESTION,
+                                                buttons = Gtk.ButtonsType.YES_NO,
+                                                flags = Gtk.DialogFlags.MODAL |
+                                                Gtk.DialogFlags.DESTROY_WITH_PARENT)
         message = _('There are changes that may be overwritten '
                     'by the new recognition.\n\n'
                     'Do you want to continue?')
@@ -614,9 +480,12 @@ class ImageReviewer_Controler:
         return response
 
     def recognizeCurrentPage(self):
+        if not self.ocr_engines:
+            self.main_window._warnNoOCREngines()
+            return
         image_reviewer = self.__getCurrentReviewer()
         if image_reviewer.selectable_boxes_area.getAllAreas() and \
-           self.__confirmOveritePossibilityByRecognition() != gtk.RESPONSE_YES:
+           self.__confirmOveritePossibilityByRecognition() != Gtk.ResponseType.YES:
                 return
         page = image_reviewer.page
         dialog = QueuedEventsProgressDialog(self.main_window.window)
@@ -629,7 +498,7 @@ class ImageReviewer_Controler:
         dialog.run()
 
     def recognizeDocument(self):
-        pages = self.source_images_selector_widget.getAllPages()
+        pages = self.pages_icon_view.getAllPages()
         dialog = QueuedEventsProgressDialog(self.main_window.window)
         items = []
         i = 1
@@ -647,7 +516,7 @@ class ImageReviewer_Controler:
             items.append((info, item))
             i += 1
         if has_changes and \
-           self.__confirmOveritePossibilityByRecognition() != gtk.RESPONSE_YES:
+           self.__confirmOveritePossibilityByRecognition() != Gtk.ResponseType.YES:
                 return
         dialog.setItemsList(items)
         dialog.run()
@@ -730,20 +599,21 @@ class ImageReviewer_Controler:
             document_generator.save()
 
     def __askPdfFromScratch(self):
-        ask_pdf_type_dialog = gtk.MessageDialog(self.main_window.window,
-                      gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
-                      buttons = gtk.BUTTONS_OK_CANCEL)
+        ask_pdf_type_dialog = Gtk.MessageDialog(self.main_window.window,
+                    Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+                      buttons = Gtk.ButtonsType.OK_CANCEL)
         ask_pdf_type_dialog.set_markup(_('What kind of PDF document do you '
                                          'wish?'))
-        pdf_from_scratch_radio = gtk.RadioButton(label= _('From scratch'))
+        pdf_from_scratch_radio = Gtk.RadioButton(label= _('From scratch'))
         pdf_from_scratch_radio.set_tooltip_text(
                                     _('Creates a new PDF from scratch.'))
-        searchable_pdf_radio = gtk.RadioButton(pdf_from_scratch_radio,
-                                               _('Searchable PDF'))
+        searchable_pdf_radio = \
+                         Gtk.RadioButton.new_from_widget(pdf_from_scratch_radio)
+        searchable_pdf_radio.set_label(_('Searchable PDF'))
         searchable_pdf_radio.set_tooltip_text(_('Creates a PDF based on '
                                                 'the images but with searchable '
                                                 'text.'))
-        vbox = gtk.VBox(True)
+        vbox = Gtk.VBox(True)
         vbox.add(pdf_from_scratch_radio)
         vbox.add(searchable_pdf_radio)
         content_area = ask_pdf_type_dialog.get_content_area()
@@ -752,7 +622,7 @@ class ImageReviewer_Controler:
 
         response = ask_pdf_type_dialog.run()
         ask_pdf_type_dialog.destroy()
-        if response == gtk.RESPONSE_CANCEL:
+        if response == Gtk.ResponseType.CANCEL:
             return (False, True)
 
         pdf_from_scratch = True
@@ -768,7 +638,7 @@ class ImageReviewer_Controler:
         image_reviewer.savePageData()
         if not project_name.endswith('.ocrf'):
             project_name += '.ocrf'
-        pages_data = self.source_images_selector_widget.getAllPages()
+        pages_data = self.pages_icon_view.getAllPages()
         project_saver = ProjectSaver(pages_data)
         project_saver.serialize(project_name)
 
@@ -776,7 +646,7 @@ class ImageReviewer_Controler:
         open_dialog = FileDialog('open', file_filters = [(_('OCRFeeder Projects'), [], ['*.ocrf'])])
         response = open_dialog.run()
         project_file = None
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             project_file = open_dialog.get_filename()
             project_loader = ProjectLoader(project_file)
             pages = project_loader.loadConfiguration()
@@ -787,8 +657,6 @@ class ImageReviewer_Controler:
                 self.__addImage(page.image_path,
                                 select_image = index == 0,
                                 page_data = page)
-                image_reviewer = self.__getCurrentReviewer()
-                image_reviewer.updatePageData(page)
         open_dialog.destroy()
         return project_file
 
@@ -796,13 +664,13 @@ class ImageReviewer_Controler:
         # Sync the current reviewer's page with its data
         self.__getCurrentReviewer().savePageData()
         export_dialog = PagesToExportDialog(title)
-        pages = self.source_images_selector_widget.getAllPages()
+        pages = self.pages_icon_view.getAllPages()
         # When there's only one document loaded or none,
         # we don't ask for the number of pages to export
         if len(pages) < 2:
             return pages
         response = export_dialog.run()
-        if response == gtk.RESPONSE_ACCEPT:
+        if response == Gtk.ResponseType.ACCEPT:
             if export_dialog.current_page_button.get_active():
                 pages = [self.__getCurrentReviewer().page]
             export_dialog.destroy()
@@ -814,22 +682,22 @@ class ImageReviewer_Controler:
     def __askForFileName(self, extension = ''):
         save_dialog = FileDialog('save')
         response = save_dialog.run()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             file_name = save_dialog.get_filename()
             if extension:
                 if not file_name.endswith(extension):
                     file_name += extension
             if os.path.isfile(file_name):
-                confirm_overwrite = gtk.MessageDialog(type = gtk.MESSAGE_QUESTION)
+                confirm_overwrite = Gtk.MessageDialog(message_type = Gtk.MessageType.QUESTION)
                 message = _('<b>A file named "%(name)s" already exists. Do you want '
                             'to replace it?</b>\n\nThe file exists in "%(dir)s". '
-                            'Replacing it will overwrite its contents.' %
-                            {'name': os.path.basename(file_name),
-                             'dir': os.path.dirname(file_name)})
+                            'Replacing it will overwrite its contents.') % {
+                             'name': os.path.basename(file_name),
+                             'dir': os.path.dirname(file_name)}
                 confirm_overwrite.set_markup(message)
-                confirm_overwrite.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-                confirm_overwrite.add_button(_('Replace'), gtk.RESPONSE_OK)
-                if confirm_overwrite.run() != gtk.RESPONSE_OK:
+                confirm_overwrite.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+                confirm_overwrite.add_button(_('Replace'), Gtk.ResponseType.OK)
+                if confirm_overwrite.run() != Gtk.ResponseType.OK:
                     file_name = None
                 confirm_overwrite.destroy()
             save_dialog.destroy()
@@ -842,10 +710,10 @@ class ImageReviewer_Controler:
         current_page = current_reviewer.page
         page_size_dialog = PageSizeDialog((current_page.width, current_page.height))
         response = page_size_dialog.run()
-        if response == gtk.RESPONSE_ACCEPT:
+        if response == Gtk.ResponseType.ACCEPT:
             size = page_size_dialog.getSize()
             if page_size_dialog.all_pages_radio.get_active():
-                for page in self.source_images_selector_widget.getAllPages():
+                for page in self.pages_icon_view.getAllPages():
                     page.setSize(size)
             else:
                 current_reviewer.page.setSize(size)
@@ -868,7 +736,7 @@ class ImageReviewer_Controler:
         unpaper_dialog = UnpaperDialog(current_reviewer,
                                     self.configuration_manager.unpaper,
                                     self.configuration_manager.TEMPORARY_FOLDER)
-        if unpaper_dialog.run() == gtk.RESPONSE_ACCEPT:
+        if unpaper_dialog.run() == Gtk.ResponseType.ACCEPT:
             unpapered_image = unpaper_dialog.getUnpaperedImage()
             current_reviewer.updateBackgroundImage(unpapered_image)
             unpaper_dialog.destroy()
@@ -879,11 +747,11 @@ class ImageReviewer_Controler:
         # remove all pages from notebook
         for index in range(self.notebook.get_n_pages()):
             self.notebook.remove_page(0)
-        self.source_images_selector_widget.clear()
+        self.pages_icon_view.clear()
         self.__updateStatusBar()
 
     def getPixbufsSorted(self):
-        return self.source_images_selector_widget.source_images_selector.getPixbufsSorted()
+        return self.pages_icon_view.source_images_selector.getPixbufsSorted()
 
     def updateFromConfiguration(self):
         for index in range(self.notebook.get_n_pages()):
@@ -931,6 +799,13 @@ class ImageReviewer_Controler:
         current_reviewer = self.__getCurrentReviewer()
         SpellCheckerDialog(self.main_window.window, current_reviewer, locale)
 
+    def updateOcrEngines(self, ocr_engines):
+        n_pages = self.notebook.get_n_pages()
+        index = 0
+        while index < n_pages:
+            reviewer = self.notebook.get_nth_page(index)
+            reviewer.updateOcrEngines(ocr_engines)
+            index += 1
 
 class Editor:
 
@@ -1022,9 +897,9 @@ class Editor:
 
     def __setDataBoxFont(self, font_button = None):
         font_button = font_button or self.box_editor.font_button
-        font_description = FontDescription(font_button.get_font_name())
+        font_description = Pango.FontDescription(font_button.get_font_name())
         self.data_box.setFontFace(font_description.get_family())
-        self.data_box.setFontSize(font_description.get_size() / SCALE)
+        self.data_box.setFontSize(font_description.get_size() / Pango.SCALE)
         self.data_box.setFontStyle(font_description.get_style())
         self.data_box.setFontWeight(font_description.get_weight())
 
@@ -1044,13 +919,16 @@ class Editor:
         language = language or self.box_editor.getLanguage()
         self.data_box.setLanguage(language)
 
-    def _onOCREngineChanged(self, combobox):
+    def _resetLanguages(self):
         index = self.box_editor.getSelectedOcrEngine()
         if index == -1:
             return
         engine = self.ocr_engines[index][0]
         self.box_editor.setAvailableLanguages(engine.getLanguages().keys())
 
+    def _onOCREngineChanged(self, combobox):
+        self._resetLanguages()
+
     def _onLanguageChanged(self, combobox):
         if self.data_box:
             self.data_box.setLanguage(self.box_editor.getLanguage())
@@ -1060,12 +938,18 @@ class Editor:
         x, y, width, height = self.data_box.updateBoundsFromBox(self.box)
         pixbuf_width = self.pixbuf.get_width()
         pixbuf_height = self.pixbuf.get_height()
-        sub_pixbuf = self.pixbuf.subpixbuf(x, y, min(width, pixbuf_width), min(height, pixbuf_height))
+        sub_pixbuf = self.pixbuf.new_subpixbuf(x, y,
+                                               min(width, pixbuf_width),
+                                               min(height, pixbuf_height))
+        sub_pixbuf.x = x
+        sub_pixbuf.y = y
+        sub_pixbuf.width = pixbuf_width
         self.data_box.setImage(sub_pixbuf)
 
     def updateOcrEngines(self, engines_list):
         engines_names = [engine.name for engine, path in engines_list]
         self.box_editor.setOcrEngines(engines_names)
+        self._resetLanguages()
 
     def __pressedImageContextButton(self, toggle_button):
         self.data_box.setType(IMAGE_TYPE)
